/**
 * <copyright>
 *
 * Copyright (c) 2006 Chaur G. Wu. All rights reserved. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * </copyright>
 */

package net.sf.model4lang.boolscript.compiler;

import java.util.*;

import javax.script.CompiledScript;
import javax.script.Invocable;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;
import javax.tools.JavaCompilerTool;
import javax.script.Bindings;

import net.sf.model4lang.boolscript.Var;
import net.sf.model4lang.boolscript.engine.BoolScriptEngine;

public class BoolCompiledScript extends CompiledScript {

	private BoolScriptEngine engine;
	private JavaCompilerTool.CompilationTask task;
	private Set<Var> varSet;
	
	public BoolCompiledScript(BoolScriptEngine engine, JavaCompilerTool.CompilationTask task, Set<Var> varSet){
		this.engine = engine;
		this.task = task;
		this.varSet = varSet;
	}
	
	@Override
	public Object eval(ScriptContext context) throws ScriptException {
		Bindings bindings = context.getBindings(ScriptContext.ENGINE_SCOPE);
		return eval(bindings);
	}

	@Override
	public Object eval(Bindings bindings) throws ScriptException {
		Invocable invocable = (Invocable) engine;
		Iterator<Var> iter = varSet.iterator();
		List valueList = new ArrayList();
		
		while (iter.hasNext()) {
			Var var = iter.next();
			String varName = var.getName();
			if (bindings.containsKey(varName)) 
				valueList.add(bindings.get(varName));
		}
		
		try {
			return invocable.invoke("eval", valueList.toArray());
		}
		catch (NoSuchMethodException e) {
			throw new ScriptException(e);
		}
	}
	
	@Override
	public Object eval() throws ScriptException {
		Bindings bindings = engine.getBindings(ScriptContext.ENGINE_SCOPE);
		return eval(bindings);
	}
	
	@Override
	public ScriptEngine getEngine() {
		return engine;
	}

}
