/**
 * <copyright>
 *
 * Copyright (c) 2006 Chaur G. Wu. All rights reserved. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * </copyright>
 */

package net.sf.model4lang.boolscript.engine;

import java.io.ByteArrayInputStream;
import java.io.Reader;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;
import java.util.LinkedHashSet;
import java.lang.reflect.Method;
import javax.script.AbstractScriptEngine;
import javax.script.Bindings;
import javax.script.Compilable;
import javax.script.CompiledScript;
import javax.script.Invocable;
import javax.script.ScriptContext;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptException;
import javax.tools.JavaCompilerTool;
import javax.tools.JavaFileObject;
import javax.tools.ToolProvider;

import net.sf.model4lang.boolscript.compiler.BoolCompiledScript;
import net.sf.model4lang.javacompiler.MemoryJavaFileManager;
import net.sf.model4lang.javacompiler.MemoryJavaFileManagerClassLoader;
import net.sf.model4lang.boolscript.parser.BoolScriptParser;
import net.sf.model4lang.boolscript.parser.ParseException;
import net.sf.model4lang.boolscript.BoolTerm;
import net.sf.model4lang.boolscript.Var;
import java.io.Writer;
import java.io.IOException;
import java.util.*;

public class BoolScriptEngine 
		extends AbstractScriptEngine
		implements Compilable, Invocable {

	private MemoryJavaFileManager fileManager;
	
	public BoolScriptEngine() {
		fileManager = new MemoryJavaFileManager();
	}
	
	public CompiledScript compile(Reader reader) throws ScriptException {
		char[] buffer = new char[255];
		StringBuilder scriptCode = new StringBuilder(); 
        
		try {
			while (reader.read(buffer) != -1)
				scriptCode.append(buffer);
		}
		catch (IOException ex) {
			throw new ScriptException(ex);
		}
		
		return compile(scriptCode.toString());
	}

	public CompiledScript compile(String script) throws ScriptException {
		
		try {
			Set<Var> varSet = getVariables(script);
			String parameters = getParameterString(varSet);
			String javaCode = compileBoolCode(script, parameters);
			JavaCompilerTool.CompilationTask task = compileJavaCode(javaCode);
			
	        if (task.getResult()) {
	        	return new BoolCompiledScript(this, task, varSet);
	        } else {
	            System.out.println("error in compiling");
	        	return null;
	        }
        }
        catch (Exception ex) {
        	throw new ScriptException(ex);
        }
	}

	private Set<Var> getVariables(String script) throws ParseException {
		byte[] bytes = script.getBytes();
		ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
		
		BoolScriptParser parser = new BoolScriptParser(inputStream, null);
		Set<Var> varSet = new LinkedHashSet<Var>();
		while (true) {
			BoolTerm term = parser.one_line();
			if (term == null) break;
			
			varSet.addAll(BoolTermEvaluator.getVariables(term));
		}
		
		return varSet;
	}
	
	private String getParameterString(Set<Var> varSet) {
		StringBuilder stringBuilder = new StringBuilder();
		Iterator<Var> iter = varSet.iterator();
		while (iter.hasNext()) {
			Var var = iter.next();
			stringBuilder.append(", boolean " + var.getName());
		}
		return stringBuilder.substring(2);		
	}
	
	private String compileBoolCode(String script, String parameters) throws ParseException {
		
		script = script.replaceAll("True", "true");
		script = script.replaceAll("False", "false");
		String[] lines = script.split("\n");
		
		StringBuilder javaCode = new StringBuilder();
		javaCode.append("package net.sf.model4lang.boolscript.generated;\n");
		javaCode.append("import java.util.*;\n");
		javaCode.append("import java.lang.reflect.*;\n");
		javaCode.append("class TempBoolClass {\n");
		javaCode.append("	public static List<Boolean> eval(" + parameters + ") {\n");
		javaCode.append("		List<Boolean> resultList = new ArrayList<Boolean>();\n"); 
		javaCode.append("		boolean result = false;\n");
		for (String line : lines) {
			if (line.length() > 0) {
				javaCode.append("result = " + line + ";\n");
				javaCode.append("resultList.add(new Boolean(result));\n");
			}
		}
		javaCode.append("		return resultList;\n");
		javaCode.append("	}\n");
		javaCode.append("}");
		
		return javaCode.toString();
	}
	
	private JavaCompilerTool.CompilationTask compileJavaCode(String javaCode) throws IOException {
		//use the Java compiler to compile the generated Java code.
		JavaCompilerTool compiler = ToolProvider.getSystemJavaCompilerTool();
		String classname = "net.sf.model4lang.boolscript.generated.TempBoolClass";
		List<String> options = new ArrayList<String>();
		options.add("-g:none");
		
        JavaFileObject inputFileObj = fileManager.getJavaFileForInput(null, classname, JavaFileObject.Kind.SOURCE);
        Writer inputWriter = inputFileObj.openWriter();
        inputWriter.write(javaCode);
        
        List<JavaFileObject> compilationUnits = new ArrayList<JavaFileObject>();
        compilationUnits.add(inputFileObj);
        return compiler.getTask(new PrintWriter(System.out), fileManager, null, options, null, compilationUnits);
   }
	
	public <T> T getInterface(Class<T> clasz) {
		// TODO Auto-generated method stub
		return null;
	}

	public <T> T getInterface(Object thiz, Class<T> clasz) {
		// TODO Auto-generated method stub
		return null;
	}

	public Object invoke(Object thiz, String name, Object... args) throws ScriptException, NoSuchMethodException {
		// TODO Auto-generated method stub
		return null;
	}

	public Object invoke(String name, Object... args) throws ScriptException, NoSuchMethodException {
		String classname = "net.sf.model4lang.boolscript.generated.TempBoolClass";
		MemoryJavaFileManagerClassLoader classLoader = new MemoryJavaFileManagerClassLoader(fileManager);
		try {
			Class compiledClass = classLoader.loadClass(classname);
			Method eval = compiledClass.getMethod(name, boolean.class, boolean.class);
			eval.setAccessible(true);
			Object finalresult = eval.invoke(null, args);
			return finalresult;
		}
		catch (ClassNotFoundException classNotFoundEx) {
			throw new NoSuchMethodException();
		}
		catch (NoSuchMethodException noSuchMethodEx) {
			throw noSuchMethodEx;
		}
		catch (Exception e) {
			throw new ScriptException(e);
		}
	}

	/* (non-Javadoc)
	 * @see javax.script.ScriptEngine#createBindings()
	 */
	public Bindings createBindings() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see javax.script.ScriptEngine#eval(java.io.Reader, javax.script.ScriptContext)
	 */
	public Object eval(Reader reader, ScriptContext context) throws ScriptException {
		char[] buffer = new char[255];
		StringBuilder scriptCode = new StringBuilder(); 
        
		try {
			while (reader.read(buffer) != -1)
				scriptCode.append(buffer);
		}
		catch (IOException ex) {
			throw new ScriptException(ex);
		}
		
        return eval(scriptCode.toString(), context);
	}

	/* (non-Javadoc)
	 * @see javax.script.ScriptEngine#eval(java.lang.String, javax.script.ScriptContext)
	 */
	public Object eval(String script, ScriptContext context) 
		throws ScriptException {
		try {
		  	return BoolTermEvaluator.evaluate(script, context);
        }
        catch (ParseException ex) {
        	throw new ScriptException(ex);
        }
	}

	/* (non-Javadoc)
	 * @see javax.script.ScriptEngine#getFactory()
	 */
	public ScriptEngineFactory getFactory() {
		return new BoolScriptEngineFactory();
	}
}
