/**
 * <copyright>
 *
 * Copyright (c) 2006 Chaur G. Wu. All rights reserved. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * </copyright>
 */

package net.sf.model4lang.boolscript.engine;

import java.io.*;
import javax.script.*;
import javax.script.http.*;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public abstract class GenericHttpScriptServlet extends GenericServlet {
	
	public abstract ScriptEngine getEngine(HttpServletRequest req);
	public abstract HttpScriptContext getContext(HttpServletRequest req, HttpServletResponse res) throws ServletException;
    public abstract void releaseEngine(ScriptEngine engine);
    public abstract Object runScript(ScriptEngine engine, Reader reader, HttpScriptContext ctxt) throws ScriptException;

    public void service(ServletRequest req, ServletResponse res)
        throws ServletException, IOException
    {
    	if(!(req instanceof HttpServletRequest))
    		throw new IllegalArgumentException("request object is not an instance of HttpServletRequest");
       
    	if(!(res instanceof HttpServletResponse))
            throw new IllegalArgumentException("response object is not an instance of HttpServletResponse");
       
        HttpServletRequest httpReq = (HttpServletRequest) req;
        HttpServletResponse httpRes = (HttpServletResponse) res;
        HttpScriptContext httpScriptCtxt = getContext(httpReq, httpRes);
        
        if (httpScriptCtxt == null) {
            httpRes.setStatus(500);
            throw new ServletException("HTTP script context is null.");
        }
        	
        //check Web Scripting configurations
        //we only perform check on script-disable here as an example.
        if(httpScriptCtxt.disableScript()) {
            httpRes.setStatus(403);
            return;
        }
        
        //perform check on the rest of configurations such as 
        //script-use-session, script-methods, allow-languages here

        ScriptEngine engine = getEngine(httpReq);
        if (engine == null) {
            httpRes.setStatus(500);
            throw new ServletException("Script engine is null.");
        }
        
        ScriptContext engineScriptCtxt = engine.getContext();
        httpScriptCtxt.setBindings(engineScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE), ScriptContext.ENGINE_SCOPE);
        httpScriptCtxt.setBindings(engineScriptCtxt.getBindings(ScriptContext.GLOBAL_SCOPE), ScriptContext.GLOBAL_SCOPE);
        httpScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE).put("javax.script.filename", httpReq.getRequestURI());
        httpScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE).put("context", httpScriptCtxt);
        httpScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE).put("servlet", this);
        httpScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE).put("request", httpScriptCtxt.getRequest());
        httpScriptCtxt.getBindings(ScriptContext.ENGINE_SCOPE).put("response", httpScriptCtxt.getResponse());
        
        try {
        	Reader srcReader = httpScriptCtxt.getScriptSource();
            if(srcReader == null) {
                httpRes.setStatus(404);
                return;
            }
        
            httpRes.setContentType("text/html");
            Object result = runScript(engine, srcReader, httpScriptCtxt); 
            
            if(result != null && httpScriptCtxt.displayResults())
                httpRes.getWriter().write(result.toString());
           
            httpRes.getWriter().flush();
            srcReader.close();
    	}
        catch(ScriptException ex) {
            httpRes.setStatus(500);
            throw new ServletException(ex);
        }
        finally {
        	releaseEngine(engine);
            httpScriptCtxt.release();
        }
    }
}
