/**
 * <copyright>
 *
 * Copyright (c) 2006 Chaur G. Wu. All rights reserved. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * </copyright>
 */

package net.sf.model4lang.javacompiler;

import java.io.*;
import java.util.*;

import javax.tools.*;
import javax.tools.JavaFileObject.Kind;
import java.lang.IllegalArgumentException;
import java.net.URI;

public final class MemoryJavaFileManager implements JavaFileManager {
    private Map<String, ByteArrayOutputStream> classNameToBytecodeMap;
    JavaFileManager fileManager;
    
    public MemoryJavaFileManager() {
    	JavaCompilerTool compiler = ToolProvider.getSystemJavaCompilerTool();
    	fileManager = compiler.getStandardFileManager(null);
    	classNameToBytecodeMap = new HashMap<String, ByteArrayOutputStream>();
    }

    public boolean classExists(String classname) {
    	return classNameToBytecodeMap.containsKey(classname);
    }
    
    public byte[] getBytecode(String classname) {
    	if (!classNameToBytecodeMap.containsKey(classname))
    		throw new IllegalArgumentException("class " + classname + " not found in in-memory java file manager.");
    	
    	ByteArrayOutputStream outStream = (ByteArrayOutputStream) classNameToBytecodeMap.get(classname);
    	return outStream.toByteArray();
    }
    
	/* (non-Javadoc)
	 * @see javax.tools.JavaFileManager#close()
	 */
	public void close() throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.tools.JavaFileManager#flush()
	 */
	public void flush() throws IOException {
	}

	public JavaFileObject getJavaFileForInput(Location location, String className, Kind kind) throws IOException {
		return new StringInputJavaFileObject(className);
	}

	public JavaFileObject getJavaFileForOutput(Location location, String className, Kind kind, FileObject sibling) throws IOException {
		JavaFileObject outObj = new ByteArrayOutputJavaFileObject(className);
		ByteArrayOutputStream outStream = (ByteArrayOutputStream) outObj.openOutputStream();
		this.classNameToBytecodeMap.put(className, outStream);
		return outObj;
	}
	
	public FileObject getFileForInput(Location location, String packageName, URI relativeName) throws IOException {
		//return null;
		return fileManager.getFileForInput(location, packageName, relativeName);
	}

	public FileObject getFileForOutput(Location location, String packageName, URI relativeName) throws IOException {
		//return null;
		return fileManager.getFileForOutput(location, packageName, relativeName);
	}

	/* (non-Javadoc)
	 * @see javax.tools.JavaFileManager#setLocation(java.lang.String, java.lang.String)
	 */
	public void setLocation(String location, String path) {
		// TODO Auto-generated method stub
	}

	public ClassLoader getClassLoader(Location location) {
		return fileManager.getClassLoader(location);
	}

	public boolean handleOption(String arg0, Iterator<String> arg1) {
		return fileManager.handleOption(arg0, arg1);
	}

	public String inferBinaryName(Location location, JavaFileObject file) {
		if (file instanceof SimpleJavaFileObject)
			return ((SimpleJavaFileObject) file).getName();
		else
			return fileManager.inferBinaryName(location, file);		
	}

	public Iterable<JavaFileObject> list(Location arg0, String arg1, Set<Kind> arg2, boolean arg3) throws IOException {
		return fileManager.list(arg0, arg1, arg2, arg3);
	}
}
